
<!-- INCLUDE simple.tpl -->

<!-- INCLUDE "simple.tpl" -->

<!-- INCLUDE 'simple.tpl' -->

<!-- INCLUDE templates/simple.tpl -->

<!-- INCLUDE "templates/simple.tpl" -->

<!-- INCLUDE 'templates/simple.tpl' -->
