   <!-- BEGIN ON -->         
Text
   <!-- END ON -->         
Text

Text



<!-- BEGIN ON -->Text<!-- END ON -->



Text

Text
<!-- BEGIN ON -->
Text
<!-- END ON -->
Text

Text<!-- BEGIN ON-->TEXT<!-- END ON -->Text

Text <!-- BEGIN ON-->TEXT<!-- END ON --> Text

Text<!-- BEGIN ON--> TEXT <!-- END ON -->Text

Text
   <!-- BEGIN ON -->         
Text
   <!-- END ON -->         